//
//  ELBluetoothManager.m
//  Elink
//
//  Created by iot_user on 2019/4/22.
//  Copyright © 2019 iot_iMac. All rights reserved.
//

#import "ELBluetoothManager.h"
#import <CoreBluetooth/CoreBluetooth.h>
#import <AILinkSecretTool/ELEncryptTool.h>

//服务UUID
static NSString *const serviceUUID = @"FFE0";//
//APP -> MCU
static NSString *const writeCharactisticUUID = @"FFE1";
//MCU -> APP
static NSString *const notifyCharactisticUUID = @"FFE2";
//APP <-> BLE 二者双向交互的UUID，同时有write和notify
static NSString *const writeAndNotifyCharactisticUUID = @"FFE3";

//握手超时时间
static NSInteger const HandShakeTimeout = 5.0;


@interface ELBluetoothManager()<CBCentralManagerDelegate,CBPeripheralDelegate>

@property (nonatomic, strong) CBCentralManager *centralManager;
//@property (nonatomic, strong) CBPeripheral *peripheral;
@property (nonatomic, strong) CBCharacteristic *notifyCharateristic;//MCU->APP
@property (nonatomic, strong) CBCharacteristic *writeCharateristic; //APP->MCU: A7
@property (nonatomic, strong) CBCharacteristic *notifiWriteCharateristic; //APP <-> BLE: A6


@property (nonatomic, strong) NSMutableArray<ELPeripheralModel *> *peripheralArray;//扫描到的外设


@property (nonatomic, assign) ELBluetoothState state;//蓝牙状态
@property (nonatomic, assign) BOOL isOn;//


///**
// 服务器是否打开蓝牙开关
// */
//@property (nonatomic, assign) BOOL serverOpen;

/**
 保存发送给蓝牙的握手指令
 */
@property (nonatomic, strong) NSData *handshake;

/**
 保存当前连接的设备对应的模型
 */
@property (nonatomic, strong) ELPeripheralModel *peripheralModel;


@end


@implementation ELBluetoothManager


+(__kindof ELBluetoothManager * _Nonnull)shareManager{
    static ELBluetoothManager *manager = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        manager = [[self alloc]init];
    });
    return manager;
}

-(instancetype)init{
    if (self = [super init]) {
        
        [self centralManager];
    }
    return self;
}

-(CBCentralManager *)centralManager{
    if (_centralManager == nil) {
        _centralManager = [[CBCentralManager alloc]initWithDelegate:self queue:dispatch_queue_create("BluetoothManagerQueue", DISPATCH_QUEUE_CONCURRENT)];
    }
    return _centralManager;
}


-(NSMutableArray<ELPeripheralModel *> *)peripheralArray{
    if (_peripheralArray == nil) {
        _peripheralArray = [[NSMutableArray alloc]init];
    }
    return _peripheralArray;
}


- (void)setNeedScanBindSysDevice:(BOOL)needScanBindSysDevice {
    _needScanBindSysDevice = needScanBindSysDevice;
}

#pragma mark ============ 自定义方法 ==============
-(void)startScan{
    
    if (self.centralManager.state == CBCentralManagerStatePoweredOn) {
        
        [self.peripheralArray removeAllObjects];//移除之前扫描的设备

        CBUUID *uuid = [CBUUID UUIDWithString:serviceUUID];//serviceUUID
        [self.centralManager scanForPeripheralsWithServices:@[uuid] options:@{CBCentralManagerScanOptionAllowDuplicatesKey:[NSNumber numberWithBool:YES]}];
        
    }else{
        NSLog(@"please open bluetooth");
    }
}

- (void)startScanAll {
    if (self.centralManager.state == CBCentralManagerStatePoweredOn) {
//        self.markGetBindDevice = YES;
        [self.peripheralArray removeAllObjects];//移除之前扫描的设备
        
        [self.centralManager scanForPeripheralsWithServices:nil options:@{CBCentralManagerScanOptionAllowDuplicatesKey:@(YES)}];
        [self respondToSelectorUpdateState:(ELBluetoothStateScaning)];
        
    }else{
        NSLog(@"please open bluetooth");
    }
}


-(void)stopScan{

    [self.peripheralArray removeAllObjects];
    [self.centralManager stopScan];
//    [self respondToSelectorUpdateState:ELBluetoothStateStopScan];
}

-(void)connectPeripheral:(ELPeripheralModel *)peripheralModel{
    [self stopScan];
    self.peripheralModel = peripheralModel;

    if (self.peripheralModel.peripheral) {
        NSLog(@"Start will connect");
        [self.centralManager connectPeripheral:self.peripheralModel.peripheral options:nil];
        [self respondToSelectorUpdateState:(ELBluetoothStateWillConnect)];
    }else{
        [self respondToSelectorUpdateState:(ELBluetoothStateConnectFail)];
    }
}

-(void)disconnectPeripheral{
    [self stopScan];
    if (self.peripheralModel.peripheral == nil || self.notifyCharateristic == nil) {
        self.centralManager = nil;
        return;
    }
    
    if (self.centralManager && self.peripheralModel) {
        
        [self.peripheralModel.peripheral setNotifyValue:NO forCharacteristic:self.notifyCharateristic];

        if (self.peripheralModel.peripheral) {
            [self.centralManager cancelPeripheralConnection:self.peripheralModel.peripheral];
        }
    }
    
//    self.centralManager = nil;
//    [self respondToSelectorUpdateState:ELBluetoothStateDidDisconnect];
}

#pragma mark ============ CBCentralManagerDelegate ==============
//获取蓝牙状态
-(void)centralManagerDidUpdateState:(CBCentralManager *)central{
    switch (central.state) {
        case CBCentralManagerStatePoweredOn:
        {
            NSLog(@"Powered On");
            break;
        }
        case CBCentralManagerStatePoweredOff:
            NSLog(@"Powered Off");
            break;
        case CBCentralManagerStateResetting:
            NSLog(@"Resetting");
            break;
        case CBCentralManagerStateUnsupported:
            NSLog(@"Unsupported");
            break;
        case CBCentralManagerStateUnknown:
            NSLog(@"Unknown");
            break;
        case CBCentralManagerStateUnauthorized:
            NSLog(@"Unauthorized");
            break;
        default:
            break;
    }
    
    if (central.state == CBCentralManagerStatePoweredOn) {
        [self startScan];
        self.isOn = YES;
        [self respondToSelectorUpdateState:ELBluetoothStateAvailable];
    }else{
        [self stopScan];
        self.isOn = NO;
        [self respondToSelectorUpdateState:ELBluetoothStateUnavailable];
    }
    
}

//扫描到设备
-(void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary<NSString *,id> *)advertisementData RSSI:(NSNumber *)RSSI{
    
    @synchronized(self) {
        
        //外围制造商数据
        NSData *manufactureData = [advertisementData valueForKey:CBAdvertisementDataManufacturerDataKey];
        //外围设备名称
        NSString *peripheralName = [advertisementData valueForKey:CBAdvertisementDataLocalNameKey];
        //服务UUID:所有AILINK设备只使用FFE0、另一个FEE0是蓝牙预留
        NSArray *serviceUUIDs = [advertisementData valueForKey:CBAdvertisementDataServiceUUIDsKey];
        
        //Company ID：496e
        Byte * byte = (Byte *)[manufactureData bytes];
        if ([serviceUUIDs.description rangeOfString:serviceUUID].length ) {
            NSLog(@"Scaned AILink bluetooth devices:%@",advertisementData.description);
            if (manufactureData.length<14) {
                //小于12个byte 说明数据有误
                return;
            }
            if (byte[0] == 0x6e && byte[1] == 0x49) {
                //判断是否是我们的AILink设备
                //广播数据解析
                ///<6e49/0001/0001/0001/aabbccddeeff>
                //mac地址
                NSString * macAddress = [self getMacAddressWithByte:manufactureData];
                //设备类型
                ELSupportDeviceType deviceType = (byte[2] << 8)+byte[3];
                //厂商id
                NSInteger vendorID = (byte[4]<<8)+byte[5];
                //产品id
                NSInteger productID = (byte[6]<<8)+byte[7];
                //macXOR
                NSUInteger len = manufactureData.length;
                NSData *macXOR = [manufactureData subdataWithRange:NSMakeRange(len-6, 6)];
                //deviceTypeXOR
                NSData *deviceTypeXOR = [manufactureData subdataWithRange:NSMakeRange(2, 2)];
                
                [self addPeriphralWithName:peripheralName withMacAddress:macAddress withRSSI:[RSSI integerValue] withPeripheral:peripheral withVendorID:vendorID withProductID:productID withDeviceType:deviceType withMacXOR:macXOR withDeviceTypeXOR:deviceTypeXOR];
            } else {
                NSLog(@"Other perpheral = %@",advertisementData.description);
            }
        }
    }
    
}


//MARK: 回调可连接设备
-(void)addPeriphralWithName:(NSString *)name withMacAddress:(NSString *)mac withRSSI:(NSInteger)rssi withPeripheral:(CBPeripheral *)peripheral withVendorID:(NSInteger)vendorID withProductID:(NSInteger)productID withDeviceType:(ELSupportDeviceType)deviceType withMacXOR:(NSData *)macXOR withDeviceTypeXOR:(NSData *)deviceTypeXOR{
    
    NSArray *array = [self.peripheralArray copy];
    for (ELPeripheralModel *model in array) {
        if ([model.macAddress isEqualToString:mac]) {
            return;
        }
    }

    ELPeripheralModel * model = [[ELPeripheralModel alloc]init];
    model.deviceName = name;
    model.macAddress = mac;
    model.rssi = rssi;
    model.peripheral = peripheral;
    model.deviceType = deviceType;
    model.vendorID = vendorID;
    model.productID = productID;
    model.macData = macXOR;
    model.deviceTypeData = deviceTypeXOR;
    [self.peripheralArray addObject:model];
//    [self.peripheralArray sortUsingFunction:bluetoothManagerPeriphelRSSIsortUsingFunction context:NULL];
    dispatch_async(dispatch_get_main_queue(), ^{
        if ([self.delegate respondsToSelector:@selector(bluetoothManagerScanedPeripherals:)]) {
            [self.delegate bluetoothManagerScanedPeripherals:[self.peripheralArray copy]];
        }

    });
}


//MARK: 获取连接类设备mac地址
-(NSString *)getMacAddressWithByte:(NSData *)data{
    ///<6e49/0001/0001/0001/aabbccddeeff>
    //6:6
    NSUInteger len = data.length;
    Byte *byte = (Byte *)[data bytes];
    NSString * macAddress = @"";
    //拿到的原始MAC小端序，需要转成大端序
    for (NSInteger i=len-1; i>=len-6; i--) {
        NSString * mac = [NSString stringWithFormat:@"%02x",byte[i]];
        if (i==len-1) {
            macAddress = [NSString stringWithFormat:@"%@",mac];
        }else{
            macAddress = [NSString stringWithFormat:@"%@:%@",macAddress,mac];
        }
    }
    macAddress = [macAddress uppercaseString];
    NSLog(@"macAddress===%@",macAddress);
    
    return macAddress;//转成大写
}

//用代理将蓝牙状态回调给外面
-(void)respondToSelectorUpdateState:(ELBluetoothState)state{
    
    dispatch_async(dispatch_get_main_queue(), ^{
        self.state = state;
        
        if ([self.delegate respondsToSelector:@selector(bluetoothManagerUpdateBleState:)]) {
            [self.delegate bluetoothManagerUpdateBleState:state];
        }
    });
    
}

//与外设连接成功
-(void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral{
    NSLog(@"Connected:%@",peripheral.name);
    self.peripheralModel.peripheral.delegate = self;
    //发现服务UUID
//    [self.peripheralModel.peripheral discoverServices:@[[CBUUID UUIDWithString:serviceUUID]]];

    [self.peripheralModel.peripheral discoverServices:nil];
    [self respondToSelectorUpdateState:(ELBluetoothStateDidConnect)];
    
//    [peripheral discoverServices:nil];
}

//与外设连接失败
-(void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error{
    NSLog(@"Connect error:%@",error);
    self.centralManager = nil;
    self.peripheralModel = nil;
    [self.peripheralArray removeAllObjects];
    [self respondToSelectorUpdateState:(ELBluetoothStateConnectFail)];
}

-(void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error{
    NSLog(@"didDisconnectPeripheral:%@",error);
    //
    self.centralManager = nil;
    //
    self.peripheralModel = nil;
    [self.peripheralArray removeAllObjects];
    [self respondToSelectorUpdateState:(ELBluetoothStateDidDisconnect)];
}

#pragma mark ============ CBPeripheralDelegate ==============
//外设发现服务回调
-(void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error{
    NSLog(@"didDiscoverServices = %@",peripheral.description);
    
    for (CBService *service in peripheral.services) {
        [self.peripheralModel.peripheral discoverCharacteristics:nil forService:service];
    }
}

//外设发现特征回调
-(void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(nonnull CBService *)service error:(nullable NSError *)error{
    NSLog(@"didDiscoverCharacteristicsForService = %@",service.description);
    
    if ([service.UUID.UUIDString containsString:serviceUUID]) {
        
        for (CBCharacteristic *characteristic in service.characteristics) {
            
            if (characteristic.properties & CBCharacteristicPropertyNotify) {
                [peripheral setNotifyValue:YES forCharacteristic:characteristic];
            }
            
            if ([characteristic.UUID.UUIDString containsString:notifyCharactisticUUID]) {
                self.notifyCharateristic = characteristic;
            }
            
            if ([characteristic.UUID.UUIDString containsString:writeCharactisticUUID]) {
                self.writeCharateristic = characteristic;
            }
            
            if ([characteristic.UUID.UUIDString containsString:writeAndNotifyCharactisticUUID]) {
                self.notifiWriteCharateristic = characteristic;
            }
            
            [self respondToSelectorUpdateState:(ELBluetoothStateDidDiscoverCharacteristics)];
        }
    }
}

-(void)peripheral:(CBPeripheral *)peripheral didUpdateNotificationStateForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error{
    
    NSLog(@"characteristic.isNotifying:%@",characteristic);

    if (characteristic.isNotifying) {
        //MARK:发送握手指令
        if ([characteristic.UUID.UUIDString containsString:writeAndNotifyCharactisticUUID]) {

            //开始握手：app主动发送握手指令
            self.handshake = [ELEncryptTool handshake];
            [self sendData:self.handshake];
            
            //app开始握手5s后，蓝牙不回复，则不是合法蓝牙模组，直接断开超时
            [self performSelector:@selector(disconnectPeripheral) withObject:nil afterDelay:HandShakeTimeout];
            
        }
        
    }else{
        //没有，断开
        [self.centralManager cancelPeripheralConnection:self.peripheralModel.peripheral];
    }

}

-(void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error{
    //NSLog(@"---didUpdateValueForCharacteristic:%@",characteristic.value);
    NSData *data  = characteristic.value;
    Byte * dataByte = (Byte *)[data bytes];
    //MARK:校验和
    if ([self.class checksum:data]) {
        
        dispatch_async(dispatch_get_main_queue(), ^{
            
            if (dataByte[0] == ELInetBleDataPackHead_A6) { //A6透传指令(明文,不需要解密)
                NSLog(@"->app(A6_1):%@",data);

                //MARK:Inet指令
                [self analyzeBLEBackA6Data:data];
                
            }
            else if (dataByte[0] == ELInetBleDataPackHead_A7){ //A7透传指令
                //NSLog(@"->app(A7加密数据):%@",data); //这是加密数据,打印没什么用
                [self analyzeMCUBackA7Data:data];
            }else {
                //透传数据
                NSLog(@"->app(透传数据):%@",data);
                if ([self.delegate respondsToSelector:@selector(bluetoothManagerReceivePassData:)]) {
                    [self.delegate bluetoothManagerReceivePassData:data];
                }

            }
        });
        
    }else{
    }
    
}



#pragma mark ============ 解密A7(MCU)原始数据并回调给子类处理 ==============
-(void)analyzeMCUBackA7Data:(NSData *)data {
    Byte * dataByte = (Byte *)[data bytes];
    if (data.length<7) {
        //数据小于7个长度，丢掉
        NSLog(@"->app: 数据小于7个长度，丢掉:%@",data);
    }else{
        
        ELSupportDeviceType deviceType = (dataByte[1]<<8) + dataByte[2];
        NSData *originData = [data subdataWithRange:NSMakeRange(4, data.length-6)];
        //对数据进行解密
        NSData *encryptData = [ELEncryptTool encryptXOR:self.peripheralModel.macData deviceTypeXOR:self.peripheralModel.deviceTypeData withXORData:originData];
        
        //回调A7解密后的透传数据：第4到第data.length-3个数据需要解密
        if ([self.delegate respondsToSelector:@selector(bluetoothManagerReceiveData:deviceType:)]) {
            [self.delegate bluetoothManagerReceiveData:encryptData deviceType:deviceType];
        }

        //以下仅仅是打印使用： A7协议是公开的(重新计算校验Byte)
        NSUInteger sum = dataByte[1] + dataByte[2] + encryptData.length;
        Byte *bytes = (Byte *)encryptData.bytes;
        for (int i = 0; i < encryptData.length; i++) {
            sum = sum + bytes[i];
        }
        Byte checksum[1] = {sum & 0xff}; //从新计算校验核: payload + 2Byte设备类型 + payload长度

        NSMutableData *dataM = [NSMutableData dataWithData:data];
        [dataM replaceBytesInRange:NSMakeRange(4, data.length-6) withBytes:encryptData.bytes]; //替换payload
        [dataM replaceBytesInRange:NSMakeRange(data.length-2, 1) withBytes:checksum]; //替换倒数第2个Byte校验和
        NSLog(@"mcu->app(A7_1): %@",dataM.copy);
        
    }
}

#pragma mark ============ 解析A6(BLE模块）回调数据 ==============
-(void)analyzeBLEBackA6Data:(NSData *)data{
    Byte * dataByte = (Byte *)[data bytes];
    //数据类型
    Byte type = dataByte[2];
    if (type == ELInetGetCmdTypeGetHandshake){ //MARK:收到ble握手数据
        //取消app握手超时
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(disconnectPeripheral) object:nil];
        
        //预留500ms给蓝牙校验app，等蓝牙握手成功后再回调验证成功
        [self performSelector:@selector(sureDidValidationPass) withObject:nil afterDelay:0.5];
        
    } else if (type == ELInetSetCmdTypeSetHandshake){ //MARK:收到蓝牙发送的校验数据，加密后发给蓝牙
        //对蓝牙返回的20Byte随机数进行加密，加密后发送给蓝牙
        NSData *sendData = [ELEncryptTool blueToothHandshakeWithData:data];
        
        [self sendData:sendData];
   } else {
       //其他A6指令
   }
    
}



-(void)sureDidValidationPass{
    
    [self respondToSelectorUpdateState:(ELBluetoothStateDidValidationPass)];
    
    //双向握手成功后，才能同步时间
    [self setMCUNowDateWithEnable:YES];
    
}

#pragma mark ============ 发送指令的2种方法 ==============

-(void)setMCUNowDateWithEnable:(BOOL)enable{
    NSDate *theDate = [NSDate date];
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    //公历
    [formatter setLocale:[NSLocale localeWithLocaleIdentifier:@"en_US_POSIX"]];
    [formatter setDateFormat:@"yyyy"];
    NSString * yearStr = [formatter stringFromDate:theDate];
    //month
    [formatter setDateFormat:@"MM"];
    NSString * monthStr = [formatter stringFromDate:theDate];
    //
    //day
    [formatter setDateFormat:@"dd"];
    NSString * dayStr = [formatter stringFromDate:theDate];
    //hour
    [formatter setDateFormat:@"HH"];
    NSString *hourStr = [formatter stringFromDate:theDate];
    //min
    [formatter setDateFormat:@"mm"];
    NSString *minStr = [formatter stringFromDate:theDate];
    //sec
    [formatter setDateFormat:@"ss"];
    NSString *secStr = [formatter stringFromDate:theDate];
    
    int year = [yearStr intValue];
    int month = [monthStr intValue];
    int day = [dayStr intValue];
    int hour = [hourStr intValue];
    int min = [minStr intValue];
    int sec = [secStr intValue];
    
    Byte dateByte[8] ={0x00};
    dateByte[0] = ELInetSetCmdTypeSetMCUNowDate;
    dateByte[1] = enable;
    dateByte[2] = year - 2000;
    dateByte[3] = month;
    dateByte[4] = day;
    dateByte[5] = hour;
    dateByte[6] = min;
    dateByte[7] = sec;
    NSData *playload = [[NSData alloc] initWithBytes:dateByte length:sizeof(dateByte)];
    [self sendCmdToMCUWithA6PayloadData:playload];
}

-(void)sendCmdToMCUWithA6PayloadData:(NSData * _Nonnull)data{
    if (data.length>16) {
        NSLog(@"The length of data sent should be less than 16 bytes");
        return;
    }else{
        ///<a6 01 69 6a 6a>
        Byte inetByte[20] = {0};
        //Inet指令:0xA6
        inetByte[0] = ELInetBleDataPackHead_A6;
        Byte *dataByte = (Byte *)[data bytes];
        NSInteger len = data.length;
        //Payload长度
        inetByte[1] = len;
        //校验和:1~n-2位，n是数据总长度
        inetByte[len+2] = inetByte[1];
        //给2+i位byte赋值
        for (int i=0; i<len; i++) {
            inetByte[i+2] = dataByte[i];
            //校验和累加
            inetByte[len+2] += dataByte[i];
        }
        //总的数据长度
        NSInteger sendLength = 2+len+2;
        //固定值
        inetByte[sendLength-1] = ELInetBleDataPackTail_6A;
        
        NSData * sendData = [[NSData alloc]initWithBytes:inetByte length:sendLength];
        
        [self sendData:sendData];
    }
    
}



-(void)sendCmdToMCUWithA7PayloadData:(NSData *)data deviceType:(ELSupportDeviceType)type{
    NSLog(@"->ble(A7_p):%@",data);

    NSData *macData = [self.peripheralModel.macData mutableCopy];
    NSData *typeData = [self.peripheralModel.deviceTypeData mutableCopy];
    
    if (data.length>14 || macData.length!=6 || typeData.length != 2) {
        return;
    }else{
        //
        Byte inetByte[20] = {0};
        inetByte[0] = ELInetBleDataPackHead_A7;
        //设备类型
        //0x0001
        //0xff00
        inetByte[1] = (type & 0xFF00)>>8;
        inetByte[2] = type & 0xFF;
        //Payload长度（最大14byte）
        NSInteger len = data.length;
        inetByte[3] = len;
        //A7000102 1122367A
        //校验和
        inetByte[len+4] = inetByte[1] + inetByte[2] +inetByte[3];

        //MARK:对A7的payload 数据进行XOR加密
        data = [ELEncryptTool encryptXOR:macData deviceTypeXOR:typeData withXORData:data];
        
        Byte *dataByte = (Byte *)[data bytes];
        for (int i=0; i<len; i++) {
            inetByte[4+i] = dataByte[i];
            inetByte[len+4] += dataByte[i];
        }


        //总数据长度
        NSInteger sendLen = 4+len+2;
        //固定值
        inetByte[sendLen-1] = ELInetBleDataPackTail_7A;
        NSData * sendData = [[NSData alloc]initWithBytes:inetByte length:sendLen];
       
        [self sendData:sendData];
    }
}

#pragma mark ============ 发指令队列(A6/A7都走这里) ==============
//MARK:发送数据
-(void)sendData:(NSData *)data{

    Byte *dataByte = (Byte *)[data bytes];
    if (dataByte[0]==ELInetBleDataPackHead_A7) {
        //MARK:发送A7数据
        if (self.writeCharateristic && self.peripheralModel.peripheral) {
            [self.peripheralModel.peripheral writeValue:data forCharacteristic:self.writeCharateristic type:(CBCharacteristicWriteWithoutResponse)];
            NSLog(@"->ble(A7) send success:%@",data);
        }else{
            NSLog(@"->ble(A7) send failure:%@",data);
        }
    }
    else if (dataByte[0] == ELInetBleDataPackHead_A6){
        //MARK:发送A6数据
        if (self.notifiWriteCharateristic && self.peripheralModel.peripheral) {
            [self.peripheralModel.peripheral writeValue:data forCharacteristic:self.notifiWriteCharateristic type:(CBCharacteristicWriteWithoutResponse)];
            NSLog(@"->ble(A6_1):%@",data);
        }else{
            NSLog(@"->ble(A6_0):%@",data);
        }
    }
    else{
        //MARK:发送透传数据
        if (self.writeCharateristic && self.peripheralModel.peripheral) {
            [self.peripheralModel.peripheral writeValue:data forCharacteristic:self.writeCharateristic type:(CBCharacteristicWriteWithoutResponse)];
            NSLog(@"->ble(Passthrough data) send success:%@",data);
        }else{
            NSLog(@"->ble(Passthrough data) send failure:%@",data);
        }
        
    }

    
}


#pragma mark ============ 校验核 ==============
//MARK：校验和
+(BOOL)checksum:(NSData *)data{
    
    if (data.length <= 3) { //至少要4个byte才解析
        return NO;
    }

    Byte *dataByte = (Byte *)[data bytes];
    
    NSUInteger location;
    if (dataByte[0] == ELInetBleDataPackHead_A6 && dataByte[1] == 0x11 && (dataByte[2] == ELInetGetCmdTypeGetHandshake || dataByte[2] == ELInetSetCmdTypeSetHandshake)) {
        //握手指令，从第0位校验到倒数第二位
        ///<a6/    112330 44f7b4d3 496731cf bfc8062b 9382ef    /92>
        location = 1;
    }else{
        //其他数据，从第0位校验到倒数第三位
        ///<a7  /000002 6a5a/  c67a>
        location = 2;
    }
    if ( data.length < (2+location) ) {
        return NO;
    }
    NSUInteger len = data.length;//数据长度
    
    Byte checksum = 0;//校验和
    //00 00 00
    for (int i = 1; i<len-location; i++) {
        checksum += dataByte[i];
    }
    if (checksum == dataByte[len-location]) {
        return YES;
    }else{
        return NO;
    }
}




@end
